@@ -73,7 +73,7 @@ gdk_colormap_get_type (void)
   static GType object_type = 0;
 
   if (!object_type) {
-    static const GTypeInfo object_info =
+    const GTypeInfo object_info =
       {
         sizeof (GdkColormapClass),
         (GBaseInitFunc) NULL,
@@ -287,9 +287,9 @@ gdk_colors_free (GdkColormap *colormap,
 }
 
 void
-gdk_colormap_free_colors (GdkColormap *colormap,
-                          GdkColor    *colors,
-                          gint         ncolors)
+gdk_colormap_free_colors (GdkColormap    *colormap,
+                          const GdkColor *colors,
+                          gint            ncolors)
 {
   GdkColormapPrivateDirectFB *private;
   gint                        i;
@@ -616,6 +616,5 @@ gdk_directfb_allocate_color_key (GdkColormap *colormap)
 
   private->info[255].ref_count++;
 }
-
 #define __GDK_COLOR_X11_C__
 #include "gdkaliasdef.c"
