@@ -530,6 +530,12 @@ gdk_keymap_have_bidi_layouts (GdkKeymap *keymap)
   return FALSE;
 }
 
+gboolean
+gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
+{
+  return ((GetKeyState (VK_CAPITAL) & 1) != 0);
+}
+
 gboolean
 gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
                                    guint          keyval,
@@ -865,3 +871,11 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 
   return tmp_keyval != GDK_VoidSymbol;
 }
+
+gboolean
+gdk_keymap_map_virtual_modifiers (GdkKeymap       *keymap,
+                                  GdkModifierType *state)
+{
+  /* FIXME: Is this the right thing to do? */
+  return TRUE;
+}
